// Author 		: F. de la Parra
// Date			: February, 2019
// Description 	: Class 'Student' keeps track of students identification
//                and academic data. 
// Version		: 1.0

package classProject;

public class Student {
	public String firstName;
	private String lastName;
	private int year;
	private Mark[] partialMarks = new Mark[3];
	
	Student(String firstName, String lastName, int year) {
		this.firstName = firstName;
		this.lastName = lastName;
		this.year = year;
		for (int i = 0; i < 3; i++) 
			partialMarks[i] = new Mark(0, false);
	}
	
	// Accessors
	
	public String getFirstName() {
		return this.firstName;
	}
	
	// Behaviour methods
	
	// Describes student in one sentence.
	public String description() {
		String allText = "Student " + this.firstName + " " + 
		                 this.lastName + " in year " + this.year;
		return allText;
	}
	
	// Computes points towards a final points mark of a given partial mark
	public int computePartialPoints(int points, int maxPoints, int weight) {
		if ((points < 0 || points > maxPoints) ||
		   (weight < 0 || weight > 100) ||
		   (points > maxPoints) ||
		   (maxPoints == 0))
			return 0;
		
		double ratio = points / (double) maxPoints;
		System.out.println("ratio is " + ratio);
		int computedPoints = (int) (ratio * weight);
		return computedPoints;
	}
	
	// Updates all 3 partial points mark towards a final points mark.
	public Mark[] updatePartialMarks(int[] points) {
		Mark[] updatedPoints = new Mark[3];
		if (points.length != 3) {
			for (int i = 0; i < 3; i++) 
				updatedPoints[i] = new Mark(-1, false);
		}
		else {
			updatedPoints[0].setPointsMark(computePartialPoints(points[0], 20, 25));
			updatedPoints[0].setUpdatedMarkFlag(true);
			updatedPoints[1].setPointsMark(computePartialPoints(points[1], 20, 25));
			updatedPoints[1].setUpdatedMarkFlag(true);
			updatedPoints[2].setPointsMark(computePartialPoints(points[2], 60, 50));
			updatedPoints[2].setUpdatedMarkFlag(true);
		}	
		return updatedPoints;
	}
	
	// Computes the final points mark for a student.
	public int computeFinalMark(Mark[] partialMarks) {
		int finalMark = 0;
		if (partialMarks.length != 3)
			return 0;
		for (int i = 0; i < partialMarks.length; i++) {
			if (partialMarks[i].getUpdatedMarkFlag())
				finalMark += partialMarks[i].getPointsMark();
			else
				return 0;
		}
		return finalMark;
	}
	
	// Checks whether all partial marks are in or not.
	public boolean isCourseCompleted(Mark[] partialMarks) {
		if (partialMarks.length != 3)
			return false;
		for (int i = 0; i < partialMarks.length; i++) {
			if (!partialMarks[i].getUpdatedMarkFlag())
				return false;
		}
		return true;
	}
}
