// Author 		: F. de la Parra
// Date			: February, 2019
// Description 	: Class 'StudentTest' implements tests for class 'Student'
//                to be run in the JUnit 5 unit-testing framework. 
// Version		: 1.0

/*
 * List of common assertions used in a test class 
 * to run in the JUnit framework (i.e. JUnit 5):
 * assertEquals(expected value, actual value, "Message")
 * assertNotEquals(expected value, actual value, "Message")
 * assertEqualsArray(expected array, actual array)
 * assertSame(expected object, actual object)
 * assertNotSame(expected object, actual object)
 * assertTrue(boolean expression)
 * assertFalse(boolean expression)
 * assertNull(reference)
 */
package classProject;

import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

class StudentTest {

	// Method 'setUpBeforeClass' runs before testing class 'Student'.
	
	@BeforeAll
	static void setUpBeforeClass() throws Exception {
		System.out.println("Settings things up before class Student.");
	}

	// Method 'tearDownAfterClass' runs after testing class 'Student'.
	
	@AfterAll
	static void tearDownAfterClass() throws Exception {
		System.out.println("Closing things down after class Student.");
	}

	// Method 'BeforeEach' runs before each tested method in class 'Student'.
	
	@BeforeEach
	void setUp() throws Exception {
		System.out.println("Settings things up before testing a method in class Student.");
	}

	// Method 'tearDown' runs after each tested method in class 'Student'.
	
	@AfterEach
	void tearDown() throws Exception {
		System.out.println("Closing things down after testing a method in class Student.");
	}

	// test1: checks the output value produced by method 'computePartialPoints',
	// in class 'Student', for selected sets of input values.
	
	@Test
	void test1() {
		System.out.println("Testing method 'computePartialPoints' in class Student");
		
		// Environment: Set object to test class 'Student'
		Student studentObject = new Student("John", "Doe", 2);
		
		// Assertions: Verify that method under test meets 'relevant' assertions
		assertEquals(0, studentObject.computePartialPoints(-15, 20, 25), "Testing negative points");
		// assertNotEquals(0, studentObject.computePartialPoints(-15, 20, 25));
		assertEquals(0, studentObject.computePartialPoints(15, -20, 25), "Testing negative max. points");
		assertEquals(0, studentObject.computePartialPoints(15, 20, -25), "Testing negative weight");
		
		assertEquals(0, studentObject.computePartialPoints(0, 20, 25), "Testing 0 points");
		assertEquals(0, studentObject.computePartialPoints(15, 0, 25), "Testing 0 max. points");
		assertEquals(0, studentObject.computePartialPoints(15, 20, 0), "Testing 0 weight");
		
		assertEquals(0, studentObject.computePartialPoints(30, 20, 25), "Testing points > maxPoints");
	}
	
	// test2: checks arrays of output values produced by method 'updatePartialMarks',
	// in class 'Student', for selected arrays of input values.
	
	@Test
	void test2() {
		System.out.println("Testing method 'updatePartialMarks' in class Student");
		
		// Environment: Set object to test class 'Student'
		Student studentObject = new Student("John", "Doe", 2);
		
		// Data: Set test data
		int[] pointsTestArray = {10, 15, 20, 30};    // Too many marks
		int[] pointsTestArray1 = {20,40};            // Insufficient marks
		
		
		// Assertions: Verify that method under test meets 'relevant' assertions
	
		// Test case: too many partial marks
		assertTrue(studentObject.updatePartialMarks(pointsTestArray)[0].getPointsMark() == -1, "Error1");
		assertTrue(studentObject.updatePartialMarks(pointsTestArray)[0].getUpdatedMarkFlag() == false, "Error2");
		assertTrue(studentObject.updatePartialMarks(pointsTestArray)[1].getPointsMark() == -1, "Error3");
		assertTrue(studentObject.updatePartialMarks(pointsTestArray)[1].getUpdatedMarkFlag() == false, "Error4");
		assertTrue(studentObject.updatePartialMarks(pointsTestArray)[2].getPointsMark() == -1, "Error5");
		assertTrue(studentObject.updatePartialMarks(pointsTestArray)[2].getUpdatedMarkFlag() == false, "Error6");
	
		// Test case: insufficient partial marks
		assertTrue(studentObject.updatePartialMarks(pointsTestArray1)[0].getPointsMark() == -1, "Error1");
		assertTrue(studentObject.updatePartialMarks(pointsTestArray1)[0].getUpdatedMarkFlag() == false, "Error2");
		assertTrue(studentObject.updatePartialMarks(pointsTestArray1)[1].getPointsMark() == -1, "Error3");
		assertTrue(studentObject.updatePartialMarks(pointsTestArray1)[1].getUpdatedMarkFlag() == false, "Error4");
		assertTrue(studentObject.updatePartialMarks(pointsTestArray1)[2].getPointsMark() == -1, "Error5");
		assertTrue(studentObject.updatePartialMarks(pointsTestArray1)[2].getUpdatedMarkFlag() == false, "Error6");
	}

	// test3: checks the output value produced by method 'computeFinalMark',
	// in class 'Student', for selected arrays of input values.
	
	@Test
	void test3() {
		System.out.println("Testing method 'computeFinalMark' in class Student");
		
		// Environment: Set object to test class 'Student'
		Student studentObject = new Student("John", "Doe", 2);
		
		// Data: Set test data
		
		// Too many marks
		Mark[] testMarks = new Mark[4];
		for (int i= 0; i < 4; i++) {
			testMarks[i] = new Mark(10*i, true);
		}
		
		// Insufficient marks
		Mark[] testMarks1 = new Mark[2];
		for (int i= 0; i < 2; i++) {
			testMarks1[i] = new Mark(10*i, true);
		}
		
		//assertArrayEquals(testMarks, testMarks1);
		
		// Set of marks containing an invalid mark.
		Mark[] testMarks2 = new Mark[3];
			testMarks2[0] = new Mark(10, true);
			testMarks2[1] = new Mark(20, false);
			testMarks2[2] = new Mark(30, true);
	
		// Assertions: Verify that method under test meets 'relevant' assertions
		assertEquals(0, studentObject.computeFinalMark(testMarks), "Testing oversized partial marks array");
		assertEquals(0, studentObject.computeFinalMark(testMarks1), "Testing undersized partial marks array");
		assertEquals(0, studentObject.computeFinalMark(testMarks2), "Testing invalid marks array");
	}
	
	// test4: checks the boolean value produced by method 'isCourseCompleted',
	// in class 'Student', for selected arrays of input values.
	
	@Test
	void test4() {
		System.out.println("Testing method 'isCourseCompleted' in class Student");
		
		// Environment: Set object to test class 'Student'
		Student studentObject = new Student("John", "Doe", 2);
		
		// Data: Set test data
		
		// Too many marks
		Mark[] testMarks = new Mark[4];
		for (int i= 0; i < 4; i++) {
			testMarks[i] = new Mark(10*i, true);
		}
				
		// Insufficient marks
		Mark[] testMarks1 = new Mark[2];
		for (int i= 0; i < 2; i++) {
			testMarks1[i] = new Mark(10*i, true);
		}
		
		// Set of marks containing an invalid mark.
		Mark[] testMarks3 = new Mark[3];
		testMarks3[0] = new Mark(10, false);
		testMarks3[1] = new Mark(20, true);
		testMarks3[2] = new Mark(30, true);
		
		// Set of marks containing an invalid mark.
		Mark[] testMarks4 = new Mark[3];
		testMarks4[0] = new Mark(10, true);
		testMarks4[1] = new Mark(20, false);
		testMarks4[2] = new Mark(30, true);
		
		// Set of marks containing an invalid mark.
		Mark[] testMarks5 = new Mark[3];
		testMarks5[0] = new Mark(10, true);
		testMarks5[1] = new Mark(20, true);
		testMarks5[2] = new Mark(30, false);
		
		// Test that generated arrays testMarks3, testMarks4 and testMarks5 are all different.
		//assertSame(testMarks3, testMarks3, "Objects are the same");
		//assertSame(testMarks3, testMarks4, "Objects are not the same");
		//assertArrayEquals(testMarks3, testMarks3);
		//assertArrayEquals(testMarks3, testMarks4);
		//assertArrayEquals(testMarks3, testMarks5);
		//assertArrayEquals(testMarks4, testMarks5);
		
		// Test outputs of method 'isCourseCompleted'.
		assertFalse(studentObject.isCourseCompleted(testMarks),"Testing oversized partial marks array");
		assertFalse(studentObject.isCourseCompleted(testMarks1), "Testing undersized partial marks array");
		assertFalse(studentObject.isCourseCompleted(testMarks3), "Testing invalid mark 1");
		assertFalse(studentObject.isCourseCompleted(testMarks4), "Testing invalid mark 2");
		assertFalse(studentObject.isCourseCompleted(testMarks5), "Testing invalid final exam mark");
	}
	
	// test5: is a dummy entry to report in a failure trace that test5 for a 'toBeDetermined' method,
	//		  in class 'Student', is missing.
	
	@Test
	void test5() {
		System.out.println("Dummy test for a 'toBeDetermined' method in class Student");
		fail("test5 for 'toBeDetermined' method in class 'Student' is not yet implemented");
	}
}
